# Nalezení žil v CT obraze pomocí dynamického programování
# Hledáme cestu z co nejjasnějších pixelů od prvního do posledního řádku obrazu
#
# Jan Kybic, 2016

import scipy              # Scientific Python
import scipy.misc         # for reading images
import pylab              # for plotting
import numpy as np        # numeric arrays

#from PIL import Image

def find_vertical(f):
  """ finds the brightest path from the top to the bottom row of grayscale image 'f' using
      dynamic programming, allowing horizontal steps of at most 1 pixel. The image values are assumed
      to be non-negative, so we are maximizing

      J=sum_{i=0}^ny f(i,p[i])

      such that |p[i]-p[i-1]|<=1
      
      Returns an array of the horizontal (column) coordinates 'p' of the best path.
  """
  ny,nx=f.shape # image size (rows, columns)
  c=np.zeros(nx)      # column costs for the previous row
  cnew=np.zeros(nx)   # column costs for the current row
  delta=np.zeros((ny,nx),dtype=np.int8) # the increments p[i]-p[i-1] for all pixels
  
  for iy in range(ny):     # first pass from the top to the bottom row 
    for ix in range(nx):   # for all columns
      cnew[ix],delta[iy,ix]=max(  # find the best way to get to (iy,ix)
        (c[ix+d]+f[iy,ix],d) for d in (-1,0,1) if ix+d<nx and ix-d>=0 )
    cnew,c=c,cnew          # exchange the working arrays

  p=np.zeros(ny,dtype=np.int)                      # here we store the result path
  p[ny-1]=np.argmax(c)                            # start at the best column
  for iy in range(ny-1,0,-1):                     # the second pass from the bottom
    p[iy-1]=p[iy]+delta[iy,p[iy]] 
  return p

#########################################################################################################

def test_vessels():
  f=scipy.misc.imread('figs/limb_vessels2.jpg') # reads to a numeric array
  pylab.figure(1)
  pylab.imshow(f,cmap=pylab.cm.gray) # show image with gray colormap

  p=find_vertical(f)                 # find the vessel
  pylab.figure(2)
  pylab.imshow(f,cmap=pylab.cm.gray) # show image with gray colormap
  pylab.plot(p,range(len(p)),'r.')
  pylab.show()

def test_vessels_and_save_imgs():
  f=scipy.misc.imread('figs/limb_vessels2.jpg') # reads to a numeric array
  pylab.imshow(f,cmap=pylab.cm.gray) # show image with gray colormap
  p=find_vertical(f)                 # find the vessel
  pylab.plot(p,range(len(p)),'r.')
  # the following lines are to save the resulting image to a file
  pylab.axis('image')                # set the correct aspect ratio
  pylab.axis('off')                  # set axis off
  pylab.gca().get_xaxis().set_visible(False)
  pylab.gca().get_yaxis().set_visible(False)
  pylab.gcf().set_size_inches((10,15))
  pylab.savefig('figs/vessels_output.png',bbox_inches='tight',pad_inches=0,dpi=100)

  
if __name__ == "__main__":  # running interactively
  test_vessels()    
